from datetime import datetime, timezone

import pytz


def datetime_to_timestamp(dt: datetime) -> int:
    """Convert datetime.datetime to integer timestamp in seconds"""
    return int(dt.timestamp())


def timestamp_to_datetime(timestamp: int) -> datetime:
    """Convert integer timestamp in seconds to datetime.datetime"""
    return datetime.fromtimestamp(timestamp, tz=timezone.utc)


def datetime_to_str(dt: datetime) -> str:
    return dt.strftime('%Y-%m-%d %H:%M:%S %z')


def now() -> datetime:
    """Return current datetime with UTC timezone set"""
    return datetime.now(tz=pytz.UTC)



import logging
import os
import traceback
from typing import Collection, Iterable


def short_exception_logger(exc_info, *_):
    """
    Display excpetion traceback in concise one-line format.
    Avoid printing long, superfluous lines of traceback, especially for multi-cause exceptions.
    """
    try:
        ex_type, ex, tb = exc_info

        traceback_ex = traceback.TracebackException(type(ex_type), ex, tb, limit=None)
        lines = list(_get_traceback_lines(traceback_ex))
        tb = ','.join(lines)
        cause = _root_cause_type(ex)

        logging.error(f'{str(ex).strip()}: cause={cause}, traceback={tb}')
    except BaseException as e:
        logging.exception(e)


def _root_cause_type(e: BaseException) -> str:
    while e.__cause__ is not None:
        e = e.__cause__
    return type(e).__name__


def log_exception(e: BaseException):
    """
    Log exception in concise one-line format containing message, exception type and short traceback
    """
    exc_info = (type(e), e, e.__traceback__)
    short_exception_logger(exc_info)


def _get_traceback_lines(t1: traceback.TracebackException) -> Iterable[str]:
    while True:
        frames: Collection[traceback.FrameSummary] = t1.stack
        for frame in frames:
            yield f'{os.path.normpath(frame.filename)}:{frame.lineno}'

        if t1.__cause__ is None:
            break
        t1 = t1.__cause__
